class Core:
    """
    Core class for Netbox API, responsible for requests to the API.
    """

    def __init__(self, netbox: object, endpoint: str):
        """
        Responsible for creating, getting, updating, and deleting resources in Netbox, obs. this class is not meant to be used directly, but to be inherited by other classes.

        Args:
            netbox: (object) The Netbox object generated by the Client class.
            endpoint: (str) The endpoint to interact with in Netbox.
        """
        self._netbox = netbox
        self._endpoint = endpoint

    def create(self, data) -> dict:
        """
        Create a new resource in Netbox using the provided data.

        Args:
            data: (list or dict): The data to be used to create the resource.

        Returns:
            dict: Returns a dictionary with the status code and the request data in JSON format: {status: 200, data: {respondse}}
        """
        if isinstance(data, dict):
            if not data['name']:
                return {'status': 400, 'data': 'Name is required.'}

            data['slug'] = self._netbox._slug(data['name'])
        elif isinstance(data, list):
            for item in data:
                item['slug'] = self._netbox._slug(item['name'])
        else:
            return {'status': 400, 'data': 'Invalid data type.'}

        response = self._netbox._request('post', f'{self._endpoint}', data)
        return response

    def get(
        self,
        id: int = None,
        name: str = None,
        tags: list = None,
        search: str = None,
        filter: str = None,
        limit: int = 1000,
    ) -> dict:
        """
        Get a resource from the endpoint on ID, name, tags, or search query.

        Args:
            id: (int): The ID of the resource to retrieve.
            name: (str): The name of the resource to retrieve.
            tags: (list): List of tags to filter resources.
            search: (str): Search query to filter resources.
            filter: (str): Filter query especified by the user in to filter resources.
            limit: (int): Maximum number of results to return. Defaults to 1000.

        Returns:
            dict: Returns a dictionary with the status code and the request data in JSON format: {status: 200, data: {respondse}}.
        """

        result = {'status': None, 'data': None}
        if id:
            response = self._netbox._request(
                'get', f'{self._endpoint}?id={id}'
            )
            return response

        if not id and name:
            id_name = self._netbox._get_id(name, f'{self._endpoint}')

            if id_name is not None:
                response = self._netbox._request(
                    'get', f'{self._endpoint}?id={id_name}'
                )
                return response
            else:
                result['status'] = 404
                return result

        else:
            if search:
                response = self._netbox._request(
                    'get', f'{self._endpoint}?q={search}&?limit={limit}'
                )

            if filter:
                response = self._netbox._request(
                    'get', f'{self._endpoint}?{filter}&?limit={limit}'
                )
            else:
                response = self._netbox._request(
                    'get', f'{self._endpoint}?limit={limit}'
                )

            if tags:
                if response['status'] != 200:
                    return response

                filtered_resources = []
                slug_tags = [self._netbox._slug(tag) for tag in tags]
                for item in response['data']['results']:
                    if item['tags']:
                        for tag in item['tags']:
                            if tag['slug'] in slug_tags:
                                filtered_resources.append(item)

                return response

            else:
                return response

    def update(self, data: list) -> dict:
        """
        Update a resource in Netbox using the provided data.
        Args:
            data: (list): The data to be used to update the resource.

        Returns:
            dict: Returns a dictionary with the status code and the request data in JSON format: {status: 200, data: {respondse}}

        """
        return self._netbox._request('put', f'{self._endpoint}', data)

    def delete(self, id: int) -> dict:
        """
        Delete a resource in Netbox based on its ID.
        Args:
            id: (int): The ID of the resource to delete.

        Returns:
            dict: Returns a dictionary with the status code and the request data in JSON format: {status: 200, data: {respondse}}
        """
        return self._netbox._request('delete', f'{self._endpoint}{id}/')
